@@ -52,7 +52,7 @@
 
 
 #define ARROW_WIDTH        (8)
-#define MAX_BLINKING_COUNT (16)
+#define MAX_BLINKING_COUNT (G_MAXUINT)
 
 enum
 {
@@ -66,21 +66,32 @@ enum
   PROP_ARROW_TYPE
 };
 
-static void     xfce_arrow_button_set_property  (GObject               *object,
-                                                 guint                  prop_id,
-                                                 const GValue          *value,
-                                                 GParamSpec            *pspec);
-static void     xfce_arrow_button_get_property  (GObject               *object,
-                                                 guint                  prop_id,
-                                                 GValue                *value,
-                                                 GParamSpec            *pspec);
-static void     xfce_arrow_button_finalize      (GObject               *object);
-static gboolean xfce_arrow_button_expose_event  (GtkWidget             *widget,
-                                                 GdkEventExpose        *event);
-static void     xfce_arrow_button_size_request  (GtkWidget             *widget,
-                                                 GtkRequisition        *requisition);
-static void     xfce_arrow_button_size_allocate (GtkWidget             *widget,
-                                                 GtkAllocation         *allocation);
+static void     xfce_arrow_button_set_property         (GObject               *object,
+                                                        guint                  prop_id,
+                                                        const GValue          *value,
+                                                        GParamSpec            *pspec);
+static void     xfce_arrow_button_get_property         (GObject               *object,
+                                                        guint                  prop_id,
+                                                        GValue                *value,
+                                                        GParamSpec            *pspec);
+static void     xfce_arrow_button_finalize             (GObject               *object);
+#if GTK_CHECK_VERSION (3, 0, 0)
+static gboolean xfce_arrow_button_draw                 (GtkWidget             *widget,
+                                                        cairo_t               *cr);
+static void     xfce_arrow_button_get_preferred_width  (GtkWidget             *widget,
+                                                        gint                  *minimum_width,
+                                                        gint                  *natural_width);
+static void     xfce_arrow_button_get_preferred_height (GtkWidget             *widget,
+                                                        gint                  *minimum_height,
+                                                        gint                  *natural_height);
+#else
+static gboolean xfce_arrow_button_expose_event         (GtkWidget             *widget,
+                                                        GdkEventExpose        *event);
+static void     xfce_arrow_button_size_request         (GtkWidget             *widget,
+                                                        GtkRequisition        *requisition);
+#endif
+static void     xfce_arrow_button_size_allocate        (GtkWidget             *widget,
+                                                        GtkAllocation         *allocation);
 
 
 
@@ -124,8 +135,14 @@ xfce_arrow_button_class_init (XfceArrowButtonClass * klass)
   gobject_class->finalize = xfce_arrow_button_finalize;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
+#if GTK_CHECK_VERSION (3, 0, 0)
+  gtkwidget_class->draw = xfce_arrow_button_draw;
+  gtkwidget_class->get_preferred_width = xfce_arrow_button_get_preferred_width;
+  gtkwidget_class->get_preferred_height = xfce_arrow_button_get_preferred_height;
+#else
   gtkwidget_class->expose_event = xfce_arrow_button_expose_event;
   gtkwidget_class->size_request = xfce_arrow_button_size_request;
+#endif
   gtkwidget_class->size_allocate = xfce_arrow_button_size_allocate;
 
   /**
@@ -176,8 +193,9 @@ xfce_arrow_button_init (XfceArrowButton *button)
   button->priv->last_relief = GTK_RELIEF_NORMAL;
 
   /* set some widget properties */
-  GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_NO_WINDOW);
-  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
+  gtk_widget_set_has_window (GTK_WIDGET (button), FALSE);
+  gtk_widget_set_can_default (GTK_WIDGET (button), FALSE);
+  gtk_widget_set_can_focus (GTK_WIDGET (button), FALSE);
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 }
 
@@ -240,6 +258,181 @@ xfce_arrow_button_finalize (GObject *object)
 
 
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static gboolean
+xfce_arrow_button_draw (GtkWidget *widget,
+                        cairo_t   *cr)
+{
+  XfceArrowButton *button = XFCE_ARROW_BUTTON (widget);
+  GtkWidget       *child;
+  gdouble          x, y, width;
+  GtkAllocation    alloc;
+  gdouble          angle;
+  GtkStyleContext *context;
+  GtkBorder        padding, border;
+  GdkRGBA          fg_rgba;
+
+  /* draw the button */
+  (*GTK_WIDGET_CLASS (xfce_arrow_button_parent_class)->draw) (widget, cr);
+
+  if (button->priv->arrow_type != GTK_ARROW_NONE
+      && gtk_widget_is_drawable (widget))
+    {
+      gtk_widget_get_allocation (widget, &alloc);
+      child = gtk_bin_get_child (GTK_BIN (widget));
+      context = gtk_widget_get_style_context (widget);
+      gtk_style_context_get_padding (context, gtk_widget_get_state_flags (widget), &padding);
+      gtk_style_context_get_border (context, gtk_widget_get_state_flags (widget), &border);
+
+      if (child != NULL
+          && gtk_widget_get_visible (child))
+        {
+          if (button->priv->arrow_type == GTK_ARROW_UP
+              || button->priv->arrow_type == GTK_ARROW_DOWN)
+            {
+              width = (gdouble) ARROW_WIDTH;
+              x = (gdouble) padding.left + border.left;
+              y = (gdouble) (alloc.height - width) / 2.0;
+            }
+          else
+            {
+              width = (gdouble) ARROW_WIDTH;
+              x = (gdouble) (alloc.width - width) / 2.0;
+              y = (gdouble) padding.top + border.top;
+            }
+        }
+      else
+        {
+          width = (gdouble) MIN (alloc.height - padding.top - padding.bottom - border.top - border.bottom,
+                                 alloc.width  - padding.left - padding.right - border.left - border.right);
+          width = (gdouble) CLAMP (width, 0.0, (gdouble) ARROW_WIDTH);
+
+          x = (gdouble) (alloc.width - width) / 2.0;
+          y = (gdouble) (alloc.height - width) / 2.0;
+        }
+
+      switch (button->priv->arrow_type)
+        {
+        case GTK_ARROW_DOWN:  angle = G_PI;
+          break;
+        case GTK_ARROW_LEFT:  angle = G_PI / 2.0 + G_PI;
+          break;
+        case GTK_ARROW_RIGHT: angle = G_PI / 2.0;
+          break;
+        default:              angle = 0.0;
+        }
+      gtk_style_context_get_color (context, gtk_widget_get_state_flags (widget), &fg_rgba);
+      gdk_cairo_set_source_rgba (cr, &fg_rgba);
+      if (width > 0)
+        gtk_render_arrow (context, cr, angle, x, y, width);
+    }
+
+  return TRUE;
+}
+
+
+
+static void
+xfce_arrow_button_get_preferred_width (GtkWidget *widget,
+                                       gint      *minimum_width,
+                                       gint      *natural_width)
+{
+  XfceArrowButton *button = XFCE_ARROW_BUTTON (widget);
+  GtkWidget       *child;
+  gint             minimum_child_width, natural_child_width;
+  GtkStyleContext *context;
+  GtkBorder        padding, border;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  if (child != NULL
+      && gtk_widget_get_visible (child))
+    {
+      /* use gtk for the widget size */
+      (*GTK_WIDGET_CLASS (xfce_arrow_button_parent_class)->get_preferred_width) (widget, &minimum_child_width, &natural_child_width);
+
+      /* reserve space for the arrow */
+      switch (button->priv->arrow_type)
+        {
+        case GTK_ARROW_UP:
+        case GTK_ARROW_DOWN:
+          natural_child_width += ARROW_WIDTH;
+          break;
+
+        default:
+          break;
+        }
+    }
+  else if (button->priv->arrow_type != GTK_ARROW_NONE)
+    {
+      /* style thickness */
+      context = gtk_widget_get_style_context (widget);
+      gtk_style_context_get_padding (context, gtk_widget_get_state_flags (widget), &padding);
+      gtk_style_context_get_border (context, gtk_widget_get_state_flags (widget), &border);
+      natural_child_width = (ARROW_WIDTH + padding.left + padding.right + border.left + border.right);
+      minimum_child_width = natural_child_width - ARROW_WIDTH;
+    }
+
+  if (minimum_width != NULL)
+    *minimum_width = minimum_child_width;
+
+  if (natural_width != NULL)
+    *natural_width = natural_child_width;
+}
+
+
+
+static void
+xfce_arrow_button_get_preferred_height (GtkWidget *widget,
+                                        gint      *minimum_height,
+                                        gint      *natural_height)
+{
+  XfceArrowButton *button = XFCE_ARROW_BUTTON (widget);
+  GtkWidget       *child;
+  gint             minimum_child_height, natural_child_height;
+  GtkStyleContext *context;
+  GtkBorder        padding, border;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  if (child != NULL
+      && gtk_widget_get_visible (child))
+    {
+      /* use gtk for the widget size */
+      (*GTK_WIDGET_CLASS (xfce_arrow_button_parent_class)->get_preferred_height) (widget, &minimum_child_height, &natural_child_height);
+
+      /* reserve space for the arrow */
+      switch (button->priv->arrow_type)
+        {
+        case GTK_ARROW_LEFT:
+        case GTK_ARROW_RIGHT:
+          natural_child_height += ARROW_WIDTH;
+          break;
+
+        default:
+          break;
+        }
+    }
+  else if (button->priv->arrow_type != GTK_ARROW_NONE)
+    {
+      /* style thickness */
+      context = gtk_widget_get_style_context (widget);
+      gtk_style_context_get_padding (context, gtk_widget_get_state_flags (widget), &padding);
+      gtk_style_context_get_border (context, gtk_widget_get_state_flags (widget), &border);
+      natural_child_height = (ARROW_WIDTH + padding.top + padding.bottom + border.top + border.bottom);
+      minimum_child_height = natural_child_height - ARROW_WIDTH;
+    }
+
+
+  if (minimum_height != NULL)
+    *minimum_height = minimum_child_height;
+
+  if (natural_height != NULL)
+    *natural_height = natural_child_height;
+}
+#endif
+
+
+
+#if !GTK_CHECK_VERSION (3, 0, 0)
 static gboolean
 xfce_arrow_button_expose_event (GtkWidget      *widget,
                                 GdkEventExpose *event)
@@ -329,6 +522,7 @@ xfce_arrow_button_size_request (GtkWidget      *widget,
       requisition->width = ARROW_WIDTH + 2 * widget->style->ythickness;
     }
 }
+#endif
 
 
 
@@ -346,10 +540,11 @@ xfce_arrow_button_size_allocate (GtkWidget     *widget,
   if (button->priv->arrow_type != GTK_ARROW_NONE)
     {
       child = gtk_bin_get_child (GTK_BIN (widget));
-      if (child != NULL && GTK_WIDGET_VISIBLE (child))
+      if (child != NULL
+          && gtk_widget_get_visible (child))
         {
           /* copy the child allocation */
-          child_allocation = child->allocation;
+          gtk_widget_get_allocation (child, &child_allocation);
 
           /* update the allocation to make space for the arrow */
           switch (button->priv->arrow_type)
@@ -527,9 +722,9 @@ xfce_arrow_button_set_blinking (XfceArrowButton *button,
         {
           /* start blinking timeout */
           button->priv->blinking_timeout_id =
-              g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 500,
-                                  xfce_arrow_button_blinking_timeout, button,
-                                  xfce_arrow_button_blinking_timeout_destroyed);
+              gdk_threads_add_timeout_full (G_PRIORITY_DEFAULT_IDLE, 500,
+                                            xfce_arrow_button_blinking_timeout, button,
+                                            xfce_arrow_button_blinking_timeout_destroyed);
         }
     }
   else if (button->priv->blinking_timeout_id != 0)
