@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2009 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -83,7 +83,7 @@ xfce_panel_plugin_provider_default_init (XfcePanelPluginProviderInterface *klass
 
 
 
-G_CONST_RETURN gchar *
+const gchar *
 xfce_panel_plugin_provider_get_name (XfcePanelPluginProvider *provider)
 {
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider), NULL);
@@ -115,12 +115,23 @@ xfce_panel_plugin_provider_set_size (XfcePanelPluginProvider *provider,
 
 
 void
-xfce_panel_plugin_provider_set_orientation (XfcePanelPluginProvider *provider,
-                                            GtkOrientation           orientation)
+xfce_panel_plugin_provider_set_mode (XfcePanelPluginProvider *provider,
+                                     XfcePanelPluginMode      mode)
 {
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
 
-  (*XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->set_orientation) (provider, orientation);
+  (*XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->set_mode) (provider, mode);
+}
+
+
+
+void
+xfce_panel_plugin_provider_set_nrows (XfcePanelPluginProvider *provider,
+                                      guint                    rows)
+{
+  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
+
+  (*XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->set_nrows) (provider, rows);
 }
 
 
@@ -246,5 +257,14 @@ xfce_panel_plugin_provider_set_locked (XfcePanelPluginProvider *provider,
 
 
 
+void
+xfce_panel_plugin_provider_ask_remove (XfcePanelPluginProvider *provider)
+{
+  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
+
+  (*XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->ask_remove) (provider);
+}
+
+
 #define __XFCE_PANEL_PLUGIN_PROVIDER_C__
 #include <libxfce4panel/libxfce4panel-aliasdef.c>
