@@ -40,6 +40,7 @@
 #include <dbus/dbus-glib.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <libwnck/libwnck.h>
 
 #include <common/panel-private.h>
 #include <common/panel-debug.h>
@@ -61,6 +62,7 @@ static gboolean   opt_version = FALSE;
 static gboolean   opt_disable_wm_check = FALSE;
 static gchar     *opt_plugin_event = NULL;
 static gchar    **opt_arguments = NULL;
+static gchar     *opt_socket_id = NULL;
 
 
 
@@ -84,6 +86,7 @@ static GOptionEntry option_entries[] =
   { "disable-wm-check", 'd', 0, G_OPTION_ARG_NONE, &opt_disable_wm_check, N_("Do not wait for a window manager on startup"), NULL },
   { "version", 'V', 0, G_OPTION_ARG_NONE, &opt_version, N_("Print version information and exit"), NULL },
   { "plugin-event", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &opt_plugin_event, NULL, NULL },
+  { "socket-id", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &opt_socket_id, NULL, NULL },
   { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_STRING_ARRAY, &opt_arguments, NULL, NULL },
   { NULL }
 };
@@ -203,6 +206,7 @@ main (gint argc, gchar **argv)
   GError           *error = NULL;
   PanelDBusService *dbus_service;
   gboolean          succeed = FALSE;
+  gboolean          remote_succeed;
   guint             i;
   const gint        signums[] = { SIGINT, SIGQUIT, SIGTERM, SIGABRT, SIGUSR1 };
   const gchar      *error_msg;
@@ -264,7 +268,7 @@ main (gint argc, gchar **argv)
   else if (opt_preferences >= 0)
     {
       /* send a signal to the running instance to show the preferences dialog */
-      succeed = panel_dbus_client_display_preferences_dialog (opt_preferences, &error);
+      succeed = panel_dbus_client_display_preferences_dialog (opt_preferences, opt_socket_id, &error);
       goto dbus_return;
     }
   else if (opt_add_items >= 0)
@@ -294,7 +298,14 @@ main (gint argc, gchar **argv)
   else if (opt_plugin_event != NULL)
     {
       /* send the plugin event to the running instance */
-      succeed = panel_dbus_client_plugin_event (opt_plugin_event, &error);
+      remote_succeed = FALSE;
+      succeed = panel_dbus_client_plugin_event (opt_plugin_event, &remote_succeed, &error);
+
+      /* the panel returns EXIT_FAILURE if the dbus event succeeds, but
+       * no suitable plugin was found on the service side */
+      if (succeed && !remote_succeed)
+        succeed = FALSE;
+
       goto dbus_return;
     }
 
@@ -328,12 +339,15 @@ main (gint argc, gchar **argv)
   for (i = 0; i < G_N_ELEMENTS (signums); i++)
     signal (signums[i], panel_signal_handler);
 
+  /* set EWMH source indication */
+  wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
+
   application = panel_application_get ();
   panel_application_load (application, opt_disable_wm_check);
 
   /* open dialog if we started from launch_panel */
   if (opt_preferences >= 0)
-    panel_preferences_dialog_show_from_id (opt_preferences);
+    panel_preferences_dialog_show_from_id (opt_preferences, opt_socket_id);
 
   gtk_main ();
 
