@@ -60,6 +60,7 @@
 
 
 static void      panel_application_finalize           (GObject                *object);
+static gboolean  panel_application_autosave_timer     (gpointer                user_data);
 static void      panel_application_plugin_move        (GtkWidget              *item,
                                                        PanelApplication       *application);
 static gboolean  panel_application_plugin_insert      (PanelApplication       *application,
@@ -124,6 +125,9 @@ struct _PanelApplication
   /* internal list of opened dialogs */
   GSList             *dialogs;
 
+  /* autosave timer for plugins */
+  guint               autosave_timer_id;
+
 #ifdef GDK_WINDOWING_X11
   guint               wait_for_wm_timeout_id;
 #endif
@@ -220,6 +224,10 @@ panel_application_init (PanelApplication *application)
 
   /* get a factory reference so it never unloads */
   application->factory = panel_module_factory_get ();
+
+  /* start the autosave timer for plugins */
+  application->autosave_timer_id = g_timeout_add_seconds (60 * 10,
+      panel_application_autosave_timer, application);
 }
 
 
@@ -231,6 +239,14 @@ panel_application_finalize (GObject *object)
 
   panel_return_if_fail (application->dialogs == NULL);
 
+  if (application->autosave_timer_id != 0)
+    {
+      g_source_remove (application->autosave_timer_id);
+
+      /* save plugins */
+      panel_application_autosave_timer (application);
+    }
+
 #ifdef GDK_WINDOWING_X11
   /* stop autostart timeout */
   if (application->wait_for_wm_timeout_id != 0)
@@ -251,6 +267,19 @@ panel_application_finalize (GObject *object)
 
 
 
+static gboolean
+panel_application_autosave_timer (gpointer user_data)
+{
+  PanelApplication *application = PANEL_APPLICATION (user_data);
+
+  /* emit a save signal for the plugins */
+  panel_application_save (application, SAVE_PLUGIN_PROVIDERS);
+
+  return TRUE;
+}
+
+
+
 static void
 panel_application_xfconf_window_bindings (PanelApplication *application,
                                           PanelWindow      *window,
@@ -830,6 +859,10 @@ panel_application_drag_data_received (PanelWindow      *window,
   guint              i;
   gboolean           found;
   gint               n_items;
+  gboolean           child_small;
+  gboolean           child_expand;
+  gboolean           child_shrink;
+  GtkWidget         *parent_itembar;
 
   panel_return_if_fail (PANEL_IS_WINDOW (window));
   panel_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
@@ -889,7 +922,8 @@ panel_application_drag_data_received (PanelWindow      *window,
           panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
 
           /* check if we move to another itembar */
-          if (gtk_widget_get_parent (provider) == itembar)
+          parent_itembar = gtk_widget_get_parent (provider);
+          if (parent_itembar == itembar)
             {
               /* get the current position on the itembar */
               old_position = panel_itembar_get_child_index (PANEL_ITEMBAR (itembar),
@@ -905,6 +939,12 @@ panel_application_drag_data_received (PanelWindow      *window,
             }
           else
             {
+              /* get properties from old itemsbar */
+              gtk_container_child_get (GTK_CONTAINER (parent_itembar), provider,
+                                       "expand", &child_expand,
+                                       "shrink", &child_shrink,
+                                       "small", &child_small, NULL);
+
               /* reparent the widget, this will also call remove and add for the itembar */
               gtk_widget_hide (provider);
               gtk_widget_reparent (provider, itembar);
@@ -913,6 +953,12 @@ panel_application_drag_data_received (PanelWindow      *window,
               /* move the item to the correct position on the itembar */
               panel_itembar_reorder_child (PANEL_ITEMBAR (itembar), provider, application->drop_index);
 
+              /* restore child properties */
+              gtk_container_child_set (GTK_CONTAINER (itembar), provider,
+                                       "expand", child_expand,
+                                       "shrink", child_shrink,
+                                       "small", child_small, NULL);
+
               /* send all the needed panel information to the plugin */
               panel_window_set_povider_info (window, provider, TRUE);
 
@@ -1437,6 +1483,7 @@ panel_application_new_window (PanelApplication *application,
   guint               i;
   gchar              *position;
   static gint         unqiue_id_counter = 1;
+  GtkWindowGroup     *window_group;
 
   panel_return_val_if_fail (PANEL_IS_APPLICATION (application), NULL);
   panel_return_val_if_fail (screen == NULL || GDK_IS_SCREEN (screen), NULL);
@@ -1454,6 +1501,11 @@ panel_application_new_window (PanelApplication *application,
   /* create panel window */
   window = panel_window_new (screen, panel_id);
 
+  /* put the window in its own group */
+  window_group = gtk_window_group_new ();
+  gtk_window_group_add_window (window_group, GTK_WINDOW (window));
+  g_object_weak_ref (G_OBJECT (window), (GWeakNotify) g_object_unref, window_group);
+
   /* add the window to internal list */
   application->windows = g_slist_append (application->windows, window);
 
