@@ -27,6 +27,7 @@
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
+#include <exo/exo.h>
 #include <common/panel-private.h>
 #include <common/panel-dbus.h>
 #include <libxfce4util/libxfce4util.h>
@@ -44,6 +45,7 @@
 static void      panel_dbus_service_finalize                   (GObject            *object);
 static gboolean  panel_dbus_service_display_preferences_dialog (PanelDBusService   *service,
                                                                 guint               active,
+                                                                const gchar        *socket_id,
                                                                 GError            **error);
 static gboolean  panel_dbus_service_display_items_dialog       (PanelDBusService   *service,
                                                                 guint               active,
@@ -59,6 +61,7 @@ static gboolean  panel_dbus_service_plugin_event               (PanelDBusService
                                                                 const gchar       *plugin_name,
                                                                 const gchar       *name,
                                                                 const GValue      *value,
+                                                                gboolean          *OUT_succeed,
                                                                 GError           **error);
 static gboolean  panel_dbus_service_terminate                  (PanelDBusService   *service,
                                                                 gboolean            restart,
@@ -190,13 +193,14 @@ panel_dbus_service_finalize (GObject *object)
 static gboolean
 panel_dbus_service_display_preferences_dialog (PanelDBusService  *service,
                                                guint              active,
+                                               const gchar       *socket_id,
                                                GError           **error)
 {
   panel_return_val_if_fail (PANEL_IS_DBUS_SERVICE (service), FALSE);
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* show the preferences dialog */
-  panel_preferences_dialog_show_from_id (active);
+  panel_preferences_dialog_show_from_id (active, socket_id);
 
   return TRUE;
 }
@@ -212,7 +216,7 @@ panel_dbus_service_display_items_dialog (PanelDBusService  *service,
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* show the items dialog */
-  panel_preferences_dialog_show_from_id (active);
+  panel_item_dialog_show_from_id (active);
 
   return TRUE;
 }
@@ -252,11 +256,11 @@ panel_dbus_service_add_new_item (PanelDBusService  *service,
 
   application = panel_application_get ();
 
+  if (arguments != NULL && exo_str_is_empty (*arguments))
+    arguments = NULL;
+
   /* add new plugin (with or without arguments) */
-  if (arguments && *arguments != NULL)
-    panel_application_add_new_item (application, plugin_name, arguments);
-  else
-    panel_application_add_new_item (application, plugin_name, NULL);
+  panel_application_add_new_item (application, NULL, plugin_name, arguments);
 
   g_object_unref (G_OBJECT (application));
 
@@ -347,6 +351,7 @@ panel_dbus_service_plugin_event (PanelDBusService  *service,
                                  const gchar       *plugin_name,
                                  const gchar       *name,
                                  const GValue      *value,
+                                 gboolean          *OUT_succeed,
                                  GError           **error)
 {
   GSList             *plugins, *li, *lnext;
@@ -354,6 +359,7 @@ panel_dbus_service_plugin_event (PanelDBusService  *service,
   PluginEvent        *event;
   guint               handle;
   gboolean            result;
+  gboolean            plugin_replied = FALSE;
 
   panel_return_val_if_fail (PANEL_IS_DBUS_SERVICE (service), FALSE);
   panel_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -391,11 +397,18 @@ panel_dbus_service_plugin_event (PanelDBusService  *service,
           g_signal_connect (G_OBJECT (li->data), "remote-event-result",
               G_CALLBACK (panel_dbus_service_plugin_event_result), service);
 
+          /* not entirely sure the event is handled, but at least suitable
+           * plugins were found */
+          plugin_replied = TRUE;
+
           /* we're going to wait until the plugin replied */
           break;
         }
       else if (result)
         {
+          /* we've handled the event */
+          plugin_replied = TRUE;
+
           /* plugin returned %TRUE, so abort the event notification */
           break;
         }
@@ -404,6 +417,10 @@ panel_dbus_service_plugin_event (PanelDBusService  *service,
   g_slist_free (plugins);
   g_object_unref (G_OBJECT (factory));
 
+  /* return status for the panel */
+  if (OUT_succeed)
+    *OUT_succeed = plugin_replied;
+
   return TRUE;
 }
 
