@@ -30,7 +30,6 @@
 #include <sys/wait.h>
 #endif
 
-#include <exo/exo.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <libxfce4util/libxfce4util.h>
@@ -51,7 +50,7 @@
 
 
 
-#define WRAPPER_BIN LIBDIR G_DIR_SEPARATOR_S "panel" G_DIR_SEPARATOR_S "wrapper"
+#define WRAPPER_BIN HELPERDIR G_DIR_SEPARATOR_S "wrapper"
 
 
 
@@ -218,7 +217,7 @@ panel_plugin_external_wrapper_get_argv (PanelPluginExternal   *external,
 
   /* setup the basic argv */
   argv = g_new0 (gchar *, argc + 1);
-  argv[PLUGIN_ARGV_0] = g_strdup (WRAPPER_BIN);
+  argv[PLUGIN_ARGV_0] = g_strjoin ("-", WRAPPER_BIN, panel_module_get_api (external->module), NULL);
   argv[PLUGIN_ARGV_FILENAME] = g_strdup (panel_module_get_filename (external->module));
   argv[PLUGIN_ARGV_UNIQUE_ID] = g_strdup_printf ("%d", external->unique_id);;
   argv[PLUGIN_ARGV_SOCKET_ID] = g_strdup_printf ("%u", gtk_socket_get_id (GTK_SOCKET (external)));;
@@ -270,8 +269,10 @@ panel_plugin_external_wrapper_set_properties (PanelPluginExternal *external,
   /* send array to the wrapper */
   g_signal_emit (G_OBJECT (external), external_signals[SET], 0, array);
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   for (i = 0; i < array->len; i++)
     g_value_array_free (g_ptr_array_index (array, i));
+  G_GNUC_END_IGNORE_DEPRECATIONS
   g_ptr_array_free (array, TRUE);
   g_value_unset (&message);
 }
