@@ -180,6 +180,9 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   dialog->bindings = NULL;
   dialog->application = panel_application_get ();
 
+  /* block autohide */
+  panel_application_windows_blocked (dialog->application, TRUE);
+
   /* load the builder data into the object */
   gtk_builder_add_from_string (GTK_BUILDER (dialog), panel_preferences_dialog_ui,
                                panel_preferences_dialog_ui_length, NULL);
@@ -187,7 +190,6 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   /* get the dialog */
   window = gtk_builder_get_object (GTK_BUILDER (dialog), "dialog");
   panel_return_if_fail (GTK_IS_WIDGET (window));
-  panel_application_take_dialog (dialog->application, GTK_WINDOW (window));
   g_signal_connect (G_OBJECT (window), "response",
       G_CALLBACK (panel_preferences_dialog_response), dialog);
 
@@ -287,6 +289,9 @@ panel_preferences_dialog_finalize (GObject *object)
   PanelPreferencesDialog *dialog = PANEL_PREFERENCES_DIALOG (object);
   GtkWidget              *itembar;
 
+  /* unblock autohide */
+  panel_application_windows_blocked (dialog->application, FALSE);
+
   /* free bindings list */
   g_slist_free (dialog->bindings);
 
@@ -925,15 +930,22 @@ static void
 panel_preferences_dialog_item_store_rebuild (GtkWidget              *itembar,
                                              PanelPreferencesDialog *dialog)
 {
-  GList       *items, *li;
-  guint        i;
-  PanelModule *module;
-  gchar       *tooltip, *display_name;
+  GList                   *items, *li;
+  guint                    i;
+  PanelModule             *module;
+  gchar                   *tooltip, *display_name;
+  XfcePanelPluginProvider *selected_provider;
+  GtkTreeIter              iter;
+  GtkTreePath             *path;
+  GObject                 *treeview;
 
   panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
   panel_return_if_fail (GTK_IS_LIST_STORE (dialog->store));
   panel_return_if_fail (PANEL_IS_ITEMBAR (itembar));
 
+  /* memorize selected item */
+  selected_provider = panel_preferences_dialog_item_get_selected (dialog, NULL);
+
   gtk_list_store_clear (dialog->store);
 
   g_signal_handlers_block_by_func (G_OBJECT (dialog->store),
@@ -972,7 +984,7 @@ panel_preferences_dialog_item_store_rebuild (GtkWidget              *itembar,
                                      xfce_panel_plugin_provider_get_unique_id (li->data));
         }
 
-      gtk_list_store_insert_with_values (dialog->store, NULL, i,
+      gtk_list_store_insert_with_values (dialog->store, &iter, i,
                                          ITEM_COLUMN_ICON_NAME,
                                          panel_module_get_icon_name (module),
                                          ITEM_COLUMN_DISPLAY_NAME,
@@ -981,6 +993,16 @@ panel_preferences_dialog_item_store_rebuild (GtkWidget              *itembar,
                                          tooltip,
                                          ITEM_COLUMN_PROVIDER, li->data, -1);
 
+      /* reconstruct selection */
+      if (selected_provider == li->data)
+        {
+          path = gtk_tree_model_get_path (GTK_TREE_MODEL (dialog->store), &iter);
+          treeview = gtk_builder_get_object (GTK_BUILDER (dialog), "item-treeview");
+          if (GTK_IS_WIDGET (treeview))
+            gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, NULL, FALSE);
+          gtk_tree_path_free (path);
+        }
+
       g_free (tooltip);
       g_free (display_name);
     }
@@ -1020,6 +1042,7 @@ panel_preferences_dialog_item_move (GtkWidget              *button,
       itembar = gtk_bin_get_child (GTK_BIN (dialog->active));
       position = panel_itembar_get_child_index (PANEL_ITEMBAR (itembar),
                                                 GTK_WIDGET (provider));
+      path = gtk_tree_model_get_path (GTK_TREE_MODEL (dialog->store), &iter_a);
 
       if (G_LIKELY (position != -1))
         {
@@ -1045,20 +1068,20 @@ panel_preferences_dialog_item_move (GtkWidget              *button,
               /* swap the items in the list */
               iter_b = iter_a;
               if (gtk_tree_model_iter_next (GTK_TREE_MODEL (dialog->store), &iter_b))
-                gtk_list_store_swap (dialog->store, &iter_a, &iter_b);
+                {
+                  gtk_list_store_swap (dialog->store, &iter_a, &iter_b);
+                  gtk_tree_path_next (path);
+                }
             }
           else
             {
               /* get the previous item in the list */
-              path = gtk_tree_model_get_path (GTK_TREE_MODEL (dialog->store), &iter_a);
               if (gtk_tree_path_prev (path))
                 {
                   /* swap the items in the list */
                   gtk_tree_model_get_iter (GTK_TREE_MODEL (dialog->store), &iter_b, path);
                   gtk_list_store_swap (dialog->store, &iter_a, &iter_b);
                 }
-
-              gtk_tree_path_free (path);
             }
 
           /* fake update the selection */
@@ -1066,7 +1089,13 @@ panel_preferences_dialog_item_move (GtkWidget              *button,
           panel_return_if_fail (GTK_IS_WIDGET (treeview));
           selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
           panel_preferences_dialog_item_selection_changed (selection, dialog);
+
+          /* make the new selected position visible if moved out of area */
+          gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path, NULL, FALSE, 0, 0);
+          gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, NULL, FALSE);
+
         }
+      gtk_tree_path_free (path);
     }
 }
 
@@ -1373,6 +1402,7 @@ panel_preferences_dialog_show_internal (PanelWindow     *active,
       gtk_window_set_screen (GTK_WINDOW (window), screen);
 
       gtk_window_present (GTK_WINDOW (window));
+      panel_application_take_dialog (dialog_singleton->application, GTK_WINDOW (window));
     }
   else
     {
