@@ -83,6 +83,8 @@ static void         panel_window_set_property               (GObject          *o
 static void         panel_window_finalize                   (GObject          *object);
 static gboolean     panel_window_expose_event               (GtkWidget        *widget,
                                                              GdkEventExpose   *event);
+static gboolean     panel_window_delete_event               (GtkWidget        *widget,
+                                                             GdkEventAny      *event);
 static gboolean     panel_window_enter_notify_event         (GtkWidget        *widget,
                                                              GdkEventCrossing *event);
 static gboolean     panel_window_leave_notify_event         (GtkWidget        *widget,
@@ -331,6 +333,7 @@ panel_window_class_init (PanelWindowClass *klass)
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->expose_event = panel_window_expose_event;
+  gtkwidget_class->delete_event = panel_window_delete_event;
   gtkwidget_class->enter_notify_event = panel_window_enter_notify_event;
   gtkwidget_class->leave_notify_event = panel_window_leave_notify_event;
   gtkwidget_class->drag_motion = panel_window_drag_motion;
@@ -838,6 +841,16 @@ end:
 
 
 
+static gboolean
+panel_window_delete_event (GtkWidget   *widget,
+                           GdkEventAny *event)
+{
+  /* do not respond to alt-f4 or any other signals */
+  return TRUE;
+}
+
+
+
 static gboolean
 panel_window_enter_notify_event (GtkWidget        *widget,
                                  GdkEventCrossing *event)
@@ -2280,6 +2293,19 @@ panel_window_set_autohide (PanelWindow *window,
 
 
 
+static void
+panel_window_menu_toggle_locked (GtkCheckMenuItem *item,
+                                 PanelWindow      *window)
+{
+  panel_return_if_fail (GTK_IS_CHECK_MENU_ITEM (item));
+  panel_return_if_fail (PANEL_IS_WINDOW (window));
+
+  g_object_set (G_OBJECT (window), "position-locked",
+      gtk_check_menu_item_get_active (item), NULL);
+}
+
+
+
 static void
 panel_window_menu_help (void)
 {
@@ -2360,6 +2386,18 @@ panel_window_menu_popup (PanelWindow *window,
       item = gtk_separator_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
       gtk_widget_show (item);
+
+      item = gtk_check_menu_item_new_with_mnemonic (_("_Lock Panel"));
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+          window->position_locked);
+      g_signal_connect (G_OBJECT (item), "toggled",
+          G_CALLBACK (panel_window_menu_toggle_locked), window);
+      gtk_widget_show (item);
+
+      item = gtk_separator_menu_item_new ();
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+      gtk_widget_show (item);
     }
 
   /* logout item */
