@@ -31,11 +31,15 @@
 
 static void     show_desktop_plugin_screen_changed          (GtkWidget              *widget,
                                                              GdkScreen              *previous_screen);
+static void     show_desktop_plugin_construct               (XfcePanelPlugin        *panel_plugin);
 static void     show_desktop_plugin_free_data               (XfcePanelPlugin        *panel_plugin);
 static gboolean show_desktop_plugin_size_changed            (XfcePanelPlugin        *panel_plugin,
                                                              gint                    size);
 static void     show_desktop_plugin_toggled                 (GtkToggleButton        *button,
                                                              ShowDesktopPlugin      *plugin);
+static gboolean show_desktop_plugin_button_release_event    (GtkToggleButton        *button,
+                                                             GdkEventButton         *event,
+                                                             ShowDesktopPlugin      *plugin);
 static void     show_desktop_plugin_showing_desktop_changed (WnckScreen             *wnck_screen,
                                                              ShowDesktopPlugin      *plugin);
 
@@ -70,6 +74,7 @@ show_desktop_plugin_class_init (ShowDesktopPluginClass *klass)
   XfcePanelPluginClass *plugin_class;
 
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
+  plugin_class->construct = show_desktop_plugin_construct;
   plugin_class->free_data = show_desktop_plugin_free_data;
   plugin_class->size_changed = show_desktop_plugin_size_changed;
 }
@@ -94,6 +99,8 @@ show_desktop_plugin_init (ShowDesktopPlugin *plugin)
   gtk_widget_set_name (button, "showdesktop-button");
   g_signal_connect (G_OBJECT (button), "toggled",
       G_CALLBACK (show_desktop_plugin_toggled), plugin);
+  g_signal_connect (G_OBJECT (button), "button-release-event",
+      G_CALLBACK (show_desktop_plugin_button_release_event), plugin);
   xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), button);
   gtk_widget_show (button);
 
@@ -104,6 +111,14 @@ show_desktop_plugin_init (ShowDesktopPlugin *plugin)
 
 
 
+static void
+show_desktop_plugin_construct (XfcePanelPlugin *panel_plugin)
+{
+  xfce_panel_plugin_set_small (panel_plugin, TRUE);
+}
+
+
+
 static void
 show_desktop_plugin_screen_changed (GtkWidget *widget,
                                     GdkScreen *previous_screen)
@@ -167,6 +182,7 @@ show_desktop_plugin_size_changed (XfcePanelPlugin *panel_plugin,
   panel_return_val_if_fail (XFCE_IS_SHOW_DESKTOP_PLUGIN (panel_plugin), FALSE);
 
   /* keep the button squared */
+  size /= xfce_panel_plugin_get_nrows (panel_plugin);
   gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, size);
 
   return TRUE;
@@ -201,6 +217,43 @@ show_desktop_plugin_toggled (GtkToggleButton   *button,
 
 
 
+static gboolean
+show_desktop_plugin_button_release_event (GtkToggleButton   *button,
+                                          GdkEventButton    *event,
+                                          ShowDesktopPlugin *plugin)
+{
+  WnckWorkspace *active_ws;
+  GList         *windows, *li;
+  WnckWindow    *window;
+
+  panel_return_val_if_fail (XFCE_IS_SHOW_DESKTOP_PLUGIN (plugin), FALSE);
+  panel_return_val_if_fail (WNCK_IS_SCREEN (plugin->wnck_screen), FALSE);
+
+  if (event->button == 2)
+    {
+      active_ws = wnck_screen_get_active_workspace (plugin->wnck_screen);
+      windows = wnck_screen_get_windows (plugin->wnck_screen);
+
+      for (li = windows; li != NULL; li = li->next)
+        {
+          window = WNCK_WINDOW (li->data);
+
+          if (wnck_window_get_workspace (window) != active_ws)
+            continue;
+
+          /* toggle the shade state */
+          if (wnck_window_is_shaded (window))
+            wnck_window_unshade (window);
+          else
+            wnck_window_shade (window);
+        }
+    }
+
+  return FALSE;
+}
+
+
+
 static void
 show_desktop_plugin_showing_desktop_changed (WnckScreen        *wnck_screen,
                                              ShowDesktopPlugin *plugin)
