@@ -89,14 +89,18 @@ wrapper_gproxy_set (DBusGProxy              *dbus_gproxy,
           xfce_panel_plugin_provider_set_size (provider, g_value_get_int (value));
           break;
 
-        case PROVIDER_PROP_TYPE_SET_ORIENTATION:
-          xfce_panel_plugin_provider_set_orientation (provider, g_value_get_int (value));
+        case PROVIDER_PROP_TYPE_SET_MODE:
+          xfce_panel_plugin_provider_set_mode (provider, g_value_get_int (value));
           break;
 
         case PROVIDER_PROP_TYPE_SET_SCREEN_POSITION:
           xfce_panel_plugin_provider_set_screen_position (provider, g_value_get_int (value));
           break;
 
+        case PROVIDER_PROP_TYPE_SET_NROWS:
+          xfce_panel_plugin_provider_set_nrows (provider, g_value_get_int (value));
+          break;
+
         case PROVIDER_PROP_TYPE_SET_LOCKED:
           xfce_panel_plugin_provider_set_locked (provider, g_value_get_boolean (value));
           break;
@@ -267,7 +271,11 @@ main (gint argc, gchar **argv)
   GError                  *error = NULL;
   const gchar             *filename;
   gint                     unique_id;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  Window                   socket_id;
+#else
   GdkNativeWindow          socket_id;
+#endif
   const gchar             *name;
   const gchar             *display_name;
   const gchar             *comment;
@@ -276,7 +284,7 @@ main (gint argc, gchar **argv)
   /* set translation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-#ifndef NDEBUG
+#ifdef G_ENABLE_DEBUG
   /* terminate the program on warnings and critical messages */
   g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
 #endif
